@@ -1,15 +1,17 @@
 #!/usr/bin/env python3
 #
 import json
+import re
 import signal
 import subprocess
-from subprocess import DEVNULL
 import sys
 from datetime import datetime
 from pathlib import Path
+from subprocess import DEVNULL
 
 from PySide6.QtWidgets import (
     QApplication,
+    QCheckBox,
     QFileDialog,
     QGridLayout,
     QLabel,
@@ -19,13 +21,30 @@ from PySide6.QtWidgets import (
     QSystemTrayIcon,
     QVBoxLayout,
     QWidget,
-    QCheckBox,
 )
 
-FULLSCREEN_SELECTED_TEXT = """Selected area: whole screen. <br/>
+from cute_sway_recorder.screen_selection import ScreenSelectionDialog
+
+FULLSCREEN_SELECTED_TEXT = """Selected area: whole screen ({screen}) <br/>
 <font color="salmon">this window will be minimized. <br/>
 click the tray icon to stop recording</font>"""
 
+PATTERN_FILE_WITH_SUFFIX = re.compile(r".*\..*")
+
+
+def available_screens():
+    """
+    Returns a list of all available outputs via `swaymsg -t get_outputs`, e.g.:
+    ['eDP-1', 'HDMI-A-1']
+    """
+    out = subprocess.check_output(["swaymsg", "-t", "get_outputs"], text=True)
+    return [o["name"] for o in json.loads(out)]
+
+
+def set_buttons_state(*btns, enabled: bool):
+    for b in btns:
+        b.setEnabled(enabled)
+
 
 def shrink_home(path: str) -> str:
     """
@@ -44,24 +63,6 @@ def select_area() -> str:
     return subprocess.run(cmd, capture_output=True).stdout.decode().strip()
 
 
-def get_screen_dimensions() -> str:
-    """
-    Get dimensions of currently focused screen via swaymsg. Returns a string in the same format as
-    `select_area()`.
-
-    If no screen if focused (?), raises ValueError
-    """
-    outputs = subprocess.check_output(["swaymsg", "-t", "get_outputs"])
-    outputs = json.loads(outputs)
-
-    for o in outputs:
-        if o["focused"] == True:
-            rect = o["rect"]
-            return f"0,0 {rect['width']}x{rect['height']}"
-
-    raise ValueError("No focused screen found with `swaymsg -t get_outputs`")
-
-
 def make_file_dst() -> str:
     """
     Create a sensible (as much as possible, you know) default name for videos that weren't given a
@@ -74,25 +75,43 @@ def make_file_dst() -> str:
     return str(Path(pathstr).expanduser().absolute())
 
 
-def start_recording(area: str, file_dst, include_audio: bool = False) -> subprocess.Popen:
+def start_recording(
+    file_dst, include_audio: bool = False, area: str = None, screen: str = None
+) -> subprocess.Popen:
     """
     Launches a `wf-recorder` process and returns a Popen object representing
     it.
 
-    `area` is a description of a rectangular area on the screen, of the
+    area is a description of a rectangular area on the screen, of the
     following format: <x>,<y> <width>x<height>
     Same output as the `slurp` command (https://github.com/emersion/slurp).
 
-    Saves the recording to `file_dst`, which is a path-like object.
+    If area is None, record the whole monitor. If there's more than one monitor, then param screen
+    decides which one to record. screen can be taken from `swaymsg -t get_outputs`.
+    e.g.: HDMI-A-1
+
+    Note that only one of area and screen should be passed, since area encompasses the screen info
+    as well (by specifying coordinates larger than the first monitor size).
+
+    Saves the recording to file_dst, which is a path-like object.
     """
+    if area and screen:
+        raise ValueError("Only one of area and screen should be passed")
+
     if file_dst is None:
         file_dst = Path().absolute()
     else:
         Path(file_dst).parent.mkdir(parents=True, exist_ok=True)
 
-    params = ["wf-recorder", "--geometry", area, "-f", file_dst]
+    params = ["wf-recorder", "-f", file_dst]
     if include_audio:
-        params.append("-a")
+        params.append("--audio")
+    if area:
+        params.append("--geometry")
+        params.append(area)
+    if screen:
+        params.append("--output")
+        params.append(screen)
     return subprocess.Popen(params)
 
 
@@ -103,16 +122,17 @@ class CuteRecorderQtApplication:
 
     def __init__(self):
         self.is_whole_screen_selected = False
+        self.selected_area = None
+        self.selected_screen = None
 
         self.app = QApplication(sys.argv)
         self.app.setApplicationDisplayName("Cute Sway Recorder")
         self.app.setDesktopFileName("cute-sway-recorder")
 
-        self.selected_area = None
         self.lbl_selected_area = QLabel("Selected area: None")
 
         self.file_dst = make_file_dst()
-        self.lbl_file_dst = QLabel("Saving to ~/Videos")
+        self.lbl_file_dst = QLabel(f"Saving to {self.file_dst}")
 
         self.recorder_proc = None
         self.lbl_is_recording = QLabel("Not recording")
@@ -128,6 +148,7 @@ class CuteRecorderQtApplication:
 
         self.btn_stop_recording = QPushButton("Stop recording")
         self.btn_stop_recording.clicked.connect(self.btn_onclick_stop_recording)
+        self.btn_stop_recording.setEnabled(False)
 
         self.btn_pick_dest = QPushButton("Pick file destination")
         self.btn_pick_dest.clicked.connect(self.btn_onclick_pick_dst)
@@ -155,24 +176,29 @@ class CuteRecorderQtApplication:
         self.window.setLayout(self.layout)
         self.window.show()
 
-        # make sure wf-recorder is avilable
-        proc = subprocess.run(["which", "wf-recorder"], stderr=DEVNULL, stdout=DEVNULL)
-        if proc.returncode != 0:
-            warning = QMessageBox(
-                QMessageBox.Critical,
-                "wf-recorder is not installed",
-                'Refer to <a href="https://github.com/ammen99/wf-recorder">their github</a> for '
-                "installation instructions :)",
-                parent=self.window,
-            )
-            warning.exec()
-            self.window.destroy()
-            sys.exit(1)
+        self.cmd_available_or_exit("wf-recorder")
+        self.cmd_available_or_exit("slurp")
 
         self.icon = QSystemTrayIcon(self.window)
         self.icon.setIcon(self.window.style().standardIcon(QStyle.SP_MediaStop))
         self.icon.activated.connect(self.tray_icon_activated_handler)
 
+    def cmd_available_or_exit(self, cmd: str):
+        """
+        Show an error popup if cmd is not available in PATH, then exit.
+        Availability is checked via `which`
+        """
+        proc = subprocess.run(["which", cmd], stderr=DEVNULL, stdout=DEVNULL)
+        if proc.returncode != 0:
+            warning = QMessageBox(
+                QMessageBox.Critical,
+                "Executable not found",
+                f"<code>{cmd}</code> not in PATH please install it :(",
+                parent=self.window,
+            )
+            warning.exec()
+            sys.exit(1)
+
     def tray_icon_activated_handler(self, reason):
         # tray icon was clicked
         if reason == QSystemTrayIcon.Trigger:
@@ -182,42 +208,76 @@ class CuteRecorderQtApplication:
 
     def btn_onclick_select_area(self):
         self.selected_area = select_area()
+        self.selected_screen = None
         self.lbl_selected_area.setText(f"Selected area: {self.selected_area}")
         self.is_whole_screen_selected = False
 
     def btn_onclick_select_whole_screen(self):
-        self.selected_area = get_screen_dimensions()
-        self.lbl_selected_area.setText(FULLSCREEN_SELECTED_TEXT)
+        screens = available_screens()
+        if len(screens) > 1:
+            selected_screen_idx = ScreenSelectionDialog(screens, parent=self.window).exec()
+            if selected_screen_idx == -1:
+                return
+            self.selected_screen = screens[selected_screen_idx]
+        self.selected_area = None
+        self.lbl_selected_area.setText(FULLSCREEN_SELECTED_TEXT.format(screen=self.selected_screen))
         self.is_whole_screen_selected = True
 
     def btn_onclick_start_recording(self):
-        if self.selected_area is None:
+        # show error message and return if no area was selected
+        if self.selected_area is None and self.selected_screen is None:
             warning = QMessageBox(
                 QMessageBox.Critical,
                 "No area selected",
-                "Kindly select an area :)",
+                "Kindly select an area or pick a screen :)",
                 parent=self.window,
             )
-            warning.open()
+            warning.exec()
             return
 
+        # show tray icon if recording fullscreen
         if self.is_whole_screen_selected:
             self.window.hide()
             self.icon.show()
 
+        # disable all buttons other than stop record
+        self.btn_stop_recording.setEnabled(True)
+        set_buttons_state(
+            self.btn_pick_dest,
+            self.btn_select_area,
+            self.btn_select_whole_screen,
+            self.btn_start_recording,
+            enabled=False,
+        )
+
         self.recorder_proc = start_recording(
-            self.selected_area, self.file_dst, include_audio=self.checkbox_use_audio.isChecked()
+            self.file_dst,
+            include_audio=self.checkbox_use_audio.isChecked(),
+            area=self.selected_area,
+            screen=self.selected_screen,
         )
         self.lbl_is_recording.setText('<font color="Red">RECORDING</font>')
         self.lbl_file_dst.setText(f"Saving as: {shrink_home(self.file_dst)}")
 
     def btn_onclick_stop_recording(self):
+        self.btn_stop_recording.setEnabled(False)
+        set_buttons_state(
+            self.btn_pick_dest,
+            self.btn_select_area,
+            self.btn_select_whole_screen,
+            self.btn_start_recording,
+            enabled=True,
+        )
         self.recorder_proc.send_signal(signal.SIGINT)
-        self.lbl_is_recording.setText("Done recording - successfuly saved!")
+        self.lbl_is_recording.setText(
+            '<font color="Green">Done recording - successfuly saved!</font>'
+        )
         self.lbl_file_dst.setText(f"Saved to: {shrink_home(self.file_dst)}")
 
     def btn_onclick_pick_dst(self):
-        dst = QFileDialog.getSaveFileName(parent=self.window)[0]
+        dst: str = QFileDialog.getSaveFileName(parent=self.window)[0]
+        if not PATTERN_FILE_WITH_SUFFIX.match(dst):
+            dst = f"{dst}.mp4"
         self.file_dst = dst
         self.lbl_file_dst.setText(f"Saving as: {shrink_home(dst)}")
 
